@@ -48,7 +48,7 @@ display_recently_played (Itdb_iTunesDB *db)
 
         if (track->recent_playcount != 0) {
             char date[30];
-            time_t track_time = itdb_time_mac_to_host (track->time_played);
+            time_t track_time = track->time_played;
 
             g_print ("%s - %s - %s:\n",
                      track->artist, track->album, track->title);
@@ -70,7 +70,9 @@ static void
 display_track (Itdb_Track *track, const char *prefix) 
 {
     g_print ("%s%s - %s - %s\n", prefix,  
-	     track->artist, track->album, track->title);
+	     track->artist ? track->artist : "(null)",
+	     track->album ? track->album : "(null)",
+	     track->title ? track->title : "(null)");
     g_print ("%s\t%s\n", prefix, track->ipod_path);
 }
 
@@ -153,15 +155,6 @@ main (int argc, char *argv[])
 	    display_playlist (playlist, "");
       }
 
-      if (error)
-      {
-	  if (error->message) {
-	      g_print ("%s\n", error->message);
-	  }
-	  g_error_free (error);
-	  error = NULL;
-      }
-
       display_recently_played (itdb);
   }
 
